// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// plugin.cpp - generic plugin operation

/**
*  Copyright (c) 2001, Will Day <willday@hpgx.net>, Tim Thelin <kndroc@hotmail.com>,
*  Pieter de Zwart <pdezwart@scu.edu>, David Mart
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*   In addition, as a special exception, <name of copyright
*   holder> gives permission to link the code of this program with
*   the FOO library (or with modified versions of FOO that use the
*   same license as FOO), and distribute linked combinations including
*   the two.  You must obey the GNU General Public License in all
*   respects for all of the code used other than FOO.  If you modify
*   this file, you may extend this exception to your version of the
*   file, but you are not obligated to do so.  If you do not wish to
*   do so, delete this exception statement from your version.
*
**/

/**
* This file is part of Log Daemon.
*
*   Log Daemon is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   Log Daemon is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Log Daemon; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/



#include <extdll.h>
#include <eiface.h>
#include <meta_api.h>
#include <string.h>


#include "LogEvents.h"
#include "StandardParser.h"
#include "LogListener.h"

#include "log_logd.h"
#include "amutil.h"

//this is used to control how these booleans start life (make the starting value true to continue to parse everything)
#define PARSE_START_VALUE FALSE
#define BLOCK_START_VALUE FALSE

//these are all the boolean switches;
BOOL block[ HIGHEST_EVENT_CODE + 1 ];
BOOL parse[ HIGHEST_EVENT_CODE + 1 ];
BOOL legal[ HIGHEST_EVENT_CODE + 1 ];
//the reason for the below indentation is because it shows the logic of the parser.
//items indentated at the same level are on the same switch
BOOL allowPlayerMessages;
    //Injury
    BOOL allowSwitchC;
        //Connect
        //Suicide
        BOOL allowSwitchA;
            //RoleChange
            //NameChange
    //Disconnect
    //Enter
    //TeamSelection
    //Kill
    //Chat
    BOOL allowSwitchT;
        BOOL allowPlayerBasedAction;
            //PvP
            //PlayerAction
        //Tell
//World
BOOL allowTeam;
    //TeamAlliance
    //TeamScore
    //TeamAction
BOOL allowSwitchL;
    //Log
    //Map
BOOL allowSwitchS;
    //Map
    //ServerName
    //ServerChat
    //Cvar
//Rcon

void loadLegalCodes()
{

    for( int x = LOWEST_EVENT_CODE; x <= HIGHEST_EVENT_CODE; x++ )
    {
        legal[ x ] = FALSE;
    }
    legal[ LMSG_CVAR ] = TRUE;
    legal[ LMSG_LOG ] = TRUE;
    legal[ LMSG_MAP ] = TRUE;
    legal[ LMSG_RCON ] = TRUE;
    legal[ LMSG_SERVER_NAME ] = TRUE;
    legal[ LMSG_SERVER_SAY ] = TRUE;
    legal[ LMSG_CONNECT ] = TRUE;
    legal[ LMSG_ENTER_GAME ] = TRUE;
    legal[ LMSG_DISCONNECT ] = TRUE;
    legal[ LMSG_SUICIDE ] = TRUE;
    legal[ LMSG_TEAM_SELECT ] = TRUE;
    legal[ LMSG_ROLE_SELECT ] = TRUE;
    legal[ LMSG_NAME_CHANGE ] = TRUE;
    legal[ LMSG_KILL ] = TRUE;
    legal[ LMSG_INJURY ] = TRUE;
    legal[ LMSG_PVP ] = TRUE;
    legal[ LMSG_PLAYER_ACTION ] = TRUE;
    legal[ LMSG_TEAM_ACTION ] = TRUE;
    legal[ LMSG_WORLD_ACTION ] = TRUE;
    legal[ LMSG_CHAT ] = TRUE;
    legal[ LMSG_TEAM_ALLIANCE ] = TRUE;
    legal[ LMSG_TEAM_SCORE ] = TRUE;
    legal[ LMSG_PRIVATE_CHAT ] = TRUE;
}

void updateSwitches()
{
    allowSwitchA =              block[ LMSG_ROLE_SELECT ] || parse[ LMSG_ROLE_SELECT ] ||
                                block[ LMSG_NAME_CHANGE ] || parse[ LMSG_NAME_CHANGE ];

    allowSwitchC =              block[ LMSG_CONNECT ] || parse[ LMSG_CONNECT ] ||
                                block[ LMSG_SUICIDE ] || parse[ LMSG_SUICIDE ] ||
                                allowSwitchA;

    allowPlayerBasedAction =    block[ LMSG_PVP ] || parse[ LMSG_PVP ] ||
                                block[ LMSG_PLAYER_ACTION ] || parse[ LMSG_PLAYER_ACTION ];

    allowSwitchT =              allowPlayerBasedAction ||
                                block[ LMSG_PRIVATE_CHAT ] || parse[ LMSG_PRIVATE_CHAT ];

    allowPlayerMessages =       block[ LMSG_INJURY ] || parse[ LMSG_INJURY ] ||
                                allowSwitchC ||
                                block[ LMSG_DISCONNECT ] || parse[ LMSG_DISCONNECT ] ||
                                block[ LMSG_ENTER_GAME ] || parse[ LMSG_ENTER_GAME ] ||
                                block[ LMSG_TEAM_SELECT ] || parse[ LMSG_TEAM_SELECT ] ||
                                block[ LMSG_KILL ] || parse[ LMSG_KILL ] ||
                                block[ LMSG_CHAT ] || parse[ LMSG_CHAT ] ||
                                allowSwitchT;

    allowTeam =                 block[ LMSG_TEAM_ALLIANCE ] || parse[ LMSG_TEAM_ALLIANCE ] ||
                                block[ LMSG_TEAM_SCORE ] || parse[ LMSG_TEAM_SCORE ] ||
                                block[ LMSG_TEAM_ACTION ] || parse[ LMSG_TEAM_ACTION ]
								;

    allowSwitchL =              block[ LMSG_LOG ] || parse[ LMSG_LOG ] ||
                                block[ LMSG_MAP ] || parse[ LMSG_MAP ];

    allowSwitchS =              block[ LMSG_MAP ] || parse[ LMSG_MAP ] ||
                                block[ LMSG_SERVER_NAME ] || parse[ LMSG_SERVER_NAME ] ||
                                block[ LMSG_SERVER_SAY ] || parse[ LMSG_SERVER_SAY ] ||
                                block[ LMSG_CVAR ] || parse[ LMSG_CVAR ];
}

void initBools()
{
    for( int x = LOWEST_EVENT_CODE; x <= HIGHEST_EVENT_CODE; x++ )
    {
        parse[ x ] = PARSE_START_VALUE;
        block[ x ] = BLOCK_START_VALUE;
    }
    updateSwitches();
}

void blockRoutineActive( int id, BOOL active )
{
    if( id < LOWEST_EVENT_CODE || id > HIGHEST_EVENT_CODE || legal[ id ] != TRUE )
	{
	    LOG_MESSAGE( PLID, "ERROR: (logd_block) not a valid log code number: %i", id );
	    return;
	}

    block[ id ] = active;

    updateSwitches();
}

void issueBlock(void)
{
    int cmdNum = CMD_ARGC();
    //we only want 2 params (since the first 1 is the command used)
	if( cmdNum != 2 )
	{
	    if( cmdNum == 1 )
	        LOG_MESSAGE( PLID, "ERROR: (logd_block) no event code given" );
        else
	        LOG_MESSAGE( PLID, "ERROR: (logd_block) too many parameters given: %s", CMD_ARGS() );
	    return;
	}

	const char* strCode = CMD_ARGV(1);
    int code = atoi( strCode );
    blockRoutineActive( code, TRUE );
}


void parseRoutineActive( int id, BOOL active )
{
    if( id < LOWEST_EVENT_CODE || id > HIGHEST_EVENT_CODE || legal[ id ] != TRUE )
	{
	    LOG_MESSAGE( PLID, "ERROR: (parseRountineActive) not a valid log code number: %i", id );
	    return;
	}

    parse[ id ] = active;

    updateSwitches();
}

char* boolToString( BOOL b )
{
    if( b == TRUE )
        return "Yes";
    else
        return "No";
}

void printStatus(void)
{
    LOG_CONSOLE( PLID, "\nLogD parser status:" );
    LOG_CONSOLE( PLID, "Event Name          Event Code   Active?  Blocking?" );
    char* name;
    char* active;
    char* blocking;

    for( int x = LOWEST_EVENT_CODE; x <= HIGHEST_EVENT_CODE; x++ )
    {
        if( legal[ x ] )
        {
            name = transName( x );
            active = boolToString( parse[ x ] );
            blocking = boolToString( block[ x ] );
            LOG_CONSOLE( PLID, UTIL_VarArgs( "%-20s%-13i%-9s%s", name, x, active, blocking) );
        }
    }

}

//this is called when MM loads us only
void parserInit(void) {
	LOG_DEBUG( 1, ( "Initializing Parser module" ) );
    REG_SVR_COMMAND("logd_status", printStatus);
    REG_SVR_COMMAND("logd_block", issueBlock);

    loadLegalCodes();

    //imagine that getting loaded is equivelent to the ending of the last map
    parserDeactivate();
}

//this is called every server activation
void parserActivate(void) {
    //we do nothing here right now
    //but init code might be here in the future
}
//this is called every server activation
void parserDeactivate(void) {
    initBools();
}












//parser vars that should stay around to save mem
char message[MAX_LOGMSG_LEN];
int mlength;
int cursor; //used as an index into message
char params[ MAX_LOGMSG_LEN ]; //we know the params must be less than the max log message length
							   // since the params are generated from the log message itself...
BOOL blockMsg;                  //used to set whether to block the recieved msg or let it through
                               //to the server


//basic containers that multiple functions can see and manipulate

//a uid should never exceded 6 digits (+null char).....
#define MAX_UID_LENGTH 7
char uid1[ MAX_UID_LENGTH ];
char uid2[ MAX_UID_LENGTH ];

//these are the actual index of players on the server between 1 and MAX_PLAYERS
int id1;
int id2;

#define MAX_STRING_LENGTH 40
char string1[ MAX_STRING_LENGTH ];
char string2[ MAX_STRING_LENGTH ];
char string3[ MAX_STRING_LENGTH ];
char string4[ MAX_STRING_LENGTH ];


/***************************
	Helper Functions
****************************/

int findForward( int start, int num, char toFind )
{
	int count = 0;
    for( int x = start; x < mlength; x++ )
    {
        if( message[ x ] == toFind )
        {
            count++;
            if( count >= num )
			{
				LOG_DEBUG( 1, ( "findForward successful. Returned: %i", x ) );
                return x;
			}
        }
    }
    LOG_DEBUG( 1, ( "findForward failed" ) );
    return -1;
}

int findBackward( int end, int num, char toFind )
{
    int count = 0;
    for( int x = end; x > -1; x-- )
    {
        if( message[ x ] == toFind )
        {
            count++;
            if( count >= num )
			{
				LOG_DEBUG( 1, ( "findBackward successful. Returned: %i", x ) );
                return x;
			}
        }
    }
	LOG_DEBUG( 1, ( "findBackward failed" ) );
    return -1;
}

void loadFirstPlayerId( int endQuote )
{
    int end = findBackward( endQuote - 1, 2, '<' ) - 1;
    int beg = findBackward( end, 1, '<' );
    for( int x = 0;; x++ )
    {
        //if weve reached the end or its the end of our uid string
        if( beg + 1 + x == end || x == MAX_UID_LENGTH - 1 )
        {
            uid1[ x ] = 0; //null terminate the string
            break;
        }
        uid1[ x ] = message[ beg + 1 + x ];
    }
	LOG_DEBUG( 2, ( "loadFirstPlayerId Found UID: %s", uid1 ) );
	id1 = GetPlayerIndex( uid1 );
	LOG_DEBUG( 1, ( "GetPlayerIndex Found ID: %i", id1 ) );
}

void loadSecondPlayerId( int begQuote )
{
    int end = findBackward( findForward( begQuote + 1, 1, '\"' ) - 1, 2, '<' ) - 1;
    int beg = findBackward( end, 1, '<' );
    for( int x = 0;; x++ )
    {
        //if weve reached the end or its the end of our uid string
        if( beg + 1 + x == end || x == MAX_UID_LENGTH - 1 )
        {
            uid2[ x ] = 0; //null terminate the string\n
            break;
        }
        uid2[ x ] = message[ beg + 1 + x ];
    }
	LOG_DEBUG( 2, ( "loadSecondPlayerId Found UID: %s", uid2 ) );
	id2 = GetPlayerIndex( uid2 );
	LOG_DEBUG( 1, ( "GetPlayerIndex Found ID: %i", id2 ) );
}




void extractStringValue( int begQuote, char* tempString )
{
   char c;
    int end = findForward( begQuote + 1, 1, '\"' );
    for( int x = 0;; x++ )
    {
        //if weve reached the end or its the end of our storing string
        if( begQuote + 1 + x == end || x == MAX_STRING_LENGTH - 1 )
        {
            tempString[ x ] = 0; //null terminate the string
            break;
        }
        c = message[ begQuote + 1 + x ];
        if( c == ' ' ) //replace spaces with '$'
            tempString[ x ] = '$';
        else
            tempString[ x ] = c;
    }
	LOG_DEBUG( 1, ( "extractStringValue successful. Returned: %s.", tempString ) );
}



/**
* Loads, if present, the optional part of log message.
* Optionals are betwen parens ( '(' and ')' )
**/
void loadEnd( int firstParen, char * s )
{
	if( firstParen == -1 )
	{
		s[ 0 ] = 0; //null terminate since given a bad starting number
		LOG_DEBUG( 1, ( "LoadEnd given -1, returning" ) );
		return;
	}
	else if( message[ firstParen ] != '(' )
	{
		s[ 0 ] = 0; //null terminate since given a bad starting number
		LOG_DEBUG( 1, ( "LoadEnd not starting on a paren, returning" ) );
		return;
    }


	int x = firstParen;
	int y = 0;
	int state = 0; //0=inside parens,1=outside parens,2=within quotes - since we either use spaces, $, or #
	LOG_DEBUG( 1, ( "LoadEnd starting char: %c\n", message[ x ] ) );
    char c = message[ x ];
	while( c != 0 )
	{
	    switch( c )
	    {
	        case '(':
	            state = 0;
	            break;
	        case ')':
	            state = 1;
	            break;
	        case ' ':
	            switch( state )
	            {
	                case 0:
        	            s[ y ] = '#';
	                    break;
	                case 1:
        	            s[ y ] = ' ';
	                    break;
	                case 2:
        	            s[ y ] = '$';
	                    break;
	            }
	            y++;
	            break;
	        case '"':
	            if( state == 0 )
	            {
	                state = 2;
	            }
	            break;
	        default:
	            s[ y ] = c;
	            y++;
	            break;
	    }
		x++;
		c = message[ x ];
	}
	s[ y ] = 0; //null terminate
	LOG_DEBUG( 1, ( "LoadEnd saved string: %s\n", s ) );
}


/*********************
		MAIN
*********************/

//the main parsing routine
void resolveLogMessage(ALERT_TYPE atype, char *szFmt, ...) {
    //return immediately if its not a log alert
	if(atype != at_logged)
		RETURN_META(MRES_IGNORED);


	va_list ap;
	va_start(ap, szFmt);
	vsnprintf(message, sizeof(message), szFmt, ap);
	va_end(ap);

	LOG_DEBUG( 2, ( "resolveLogMessage.  Message: %s", message ) );

	mlength = strlen( message );
	cursor = 0;   // represents an index into the message
	blockMsg = FALSE;

	//switch statements are used beacuse they provide the fastest
	//conditional branching
    switch( message[ cursor ] )
    {
        case '"':
            if( !allowPlayerMessages )
                break;

            cursor = findForward( 1, 1, '\"' ); //scan to the end of the id string set
            loadFirstPlayerId( cursor );

            switch( message[ cursor + 2 ] )
            {
                case 'a':
                    blockMsg = block[ LMSG_INJURY ];
                    if( !parse[ LMSG_INJURY ] )
                        break;

                    //Injury message
                    loadSecondPlayerId( cursor + 11 );
                    cursor = findForward( cursor + 12, 2, '\"' );
                    extractStringValue( cursor, string1 ); //the gun
					loadEnd( findForward( cursor + 1, 1, '(' ) , string2 ); //amount of damage
					notifyListeners( LMSG_INJURY, strcpy( params, UTIL_VarArgs("%i %i %s %s", id1, id2, string1, string2 ) ) );
                    break;

                case 'c':
                    if( !allowSwitchC )
                        break;

                    switch( message[ cursor + 4 ] )
                    {
                        case 'n':
                            //connection
							blockMsg = block[ LMSG_CONNECT ];
							if( !parse[ LMSG_CONNECT ] )
                                break;

                            extractStringValue( cursor + 21, string1 ); // ip_port
		                    notifyListeners( LMSG_CONNECT, strcpy( params, UTIL_VarArgs("%i %s", id1, string1 ) ) );
                            break;

                        case 'm':
                            //suicide
							blockMsg = block[ LMSG_SUICIDE ];
							if( !parse[ LMSG_SUICIDE ] )
                                break;

                            extractStringValue( cursor + 25, string1 ); //weapon
		                    notifyListeners( LMSG_SUICIDE, strcpy( params, UTIL_VarArgs("%i %s", id1, string1 ) ) );
                            break;

                        case 'a':
                            if( !allowSwitchA )
                                break;

                            switch( message[ cursor + 10 ] )
                            {
                                case 'r':
                                    //role change
									blockMsg = block[ LMSG_ROLE_SELECT ];
									if( !parse[ LMSG_ROLE_SELECT ] )
                                        break;

                                    extractStringValue( cursor + 18, string1 ); //role
        		                    notifyListeners( LMSG_ROLE_SELECT, strcpy( params, UTIL_VarArgs("%i %s", id1, string1 ) ) );
                                    break;

                                case 'n':
                                    //name change
									blockMsg = block[ LMSG_NAME_CHANGE ];
									if( !parse[ LMSG_NAME_CHANGE ] )
                                        break;

                                    extractStringValue( cursor + 18, string1 ); //new name
          		                    notifyListeners( LMSG_NAME_CHANGE, strcpy( params, UTIL_VarArgs("%i %s", id1, string1 ) ) );
                                    break;

                                default:
                                {
                                    LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'a' Switch: %c > %s", message[ cursor + 10 ], message ) );
                                }
                            }   //end 'a' switch
                            break;

                        default:
                        {
                            LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'c' Switch: %c > %s", message[ cursor + 4 ], message ) );
                        }
                    }   //end 'c' switch
                    break;

                case 'd':
                    //disconnect
                    blockMsg = block[ LMSG_DISCONNECT ];
                    if( !parse[ LMSG_DISCONNECT ] )
                        break;

                    notifyListeners( LMSG_DISCONNECT, strcpy( params, UTIL_VarArgs("%i", id1 ) ) );
                    break;

                case 'e':
                    //entered game
                    blockMsg = block[ LMSG_ENTER_GAME ];
                    if( !parse[ LMSG_ENTER_GAME ] )
                        break;

                    notifyListeners( LMSG_ENTER_GAME, strcpy( params, UTIL_VarArgs("%i", id1 ) ) );
                    break;

                case 'j':
                    //team selection
                    blockMsg = block[ LMSG_TEAM_SELECT ];
                    if( !parse[ LMSG_TEAM_SELECT ] )
                        break;

                    extractStringValue( cursor + 14, string1 ); //the new team
                    notifyListeners( LMSG_TEAM_SELECT, strcpy( params, UTIL_VarArgs("%i %s", id1, string1 ) ) );
                    break;

                case 'k':
                    //kill
                    blockMsg = block[ LMSG_KILL ];
                    if( !parse[ LMSG_KILL ] )
                        break;

                    loadSecondPlayerId( cursor + 9 );
                    cursor = findForward( cursor + 10, 1, '\"' );
                    extractStringValue( cursor + 7, string1 ); //weapon
                    loadEnd( findForward( cursor + 8, 1, '(' ), string2 );
                    notifyListeners( LMSG_KILL, strcpy( params, UTIL_VarArgs("%i %i %s %s", id1, id2, string1, string2 ) ) );
                    break;

                case 's':
                    //chat
                    blockMsg = block[ LMSG_CHAT ];
                    if( !parse[ LMSG_CHAT ] )
                        break;

                    extractStringValue( findForward( cursor + 5, 1, '\"' ), string1 );
                    switch( message[ cursor + 5 ] )
                    {
                        case '_':
                            notifyListeners( LMSG_CHAT, strcpy( params, UTIL_VarArgs("%i %c %s", id1, 't', string1 ) ) );
                            break;
                        case ' ':
                            notifyListeners( LMSG_CHAT, strcpy( params, UTIL_VarArgs("%i %c %s", id1, 'n', string1 ) ) );
                            break;
                        default:
                        {
                            LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'Chat' Switch: %c > %s", message[ cursor + 5 ], message ) );
                        }
                    }
                    break;

                case 't':
                    if( !allowSwitchT )
                        break;

                    switch( message[ cursor + 3 ] )
                    {
                        case 'r':
                            if( !allowPlayerBasedAction )
                                break;

                            //some kind of player action
                            // so go ahead and extract the action
                            extractStringValue( cursor + 12, string1 ); //the player action
                            cursor = findForward( cursor + 13, 1, '\"' );

                            //if the message is long enough to be
                            // a PvP and pos+2 is an a, then it is a PvP
                            if( cursor + 2 < mlength - 1 &&  message[ cursor + 2 ] == 'a' )
                            {
                                //Player vs Player action
								blockMsg = block[ LMSG_PVP ];
								if( !parse[ LMSG_PVP ] )
                                    break;

                                loadSecondPlayerId( cursor + 10 ); //the person the action was against
                                notifyListeners( LMSG_PVP, strcpy( params, UTIL_VarArgs("%i %i %s", id1, id2, string1 ) ) );
                            }
                            else
                            {
                                //player action
								blockMsg = block[ LMSG_PLAYER_ACTION ];
								if( !parse[ LMSG_PLAYER_ACTION ] )
                                    break;

                                notifyListeners( LMSG_PLAYER_ACTION, strcpy( params, UTIL_VarArgs("%i %s", id1, string1 ) ) );
                            }
                            break;

                        case 'e':
                            //tell
							blockMsg = block[ LMSG_PRIVATE_CHAT ];
							if( !parse[ LMSG_PRIVATE_CHAT ] )
                                break;

                            loadSecondPlayerId( cursor + 7 );
                            extractStringValue( findForward( cursor + 8, 2, '\"' ), string1 );
                            notifyListeners( LMSG_PRIVATE_CHAT, strcpy( params, UTIL_VarArgs("%i %i %s", id1, id2, string1 ) ) );
                            break;

                        default:
                        {
                            LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 't' Switch: %c > %s", message[ cursor + 3 ], message ) );
                        }
                    }
                    break;

                default:
                {
                    LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'quote' Switch: %c > %s", message[ cursor + 2 ], message ) );
                }
            }
            break;


        case 'W':
            //world trigger
			blockMsg = block[ LMSG_WORLD_ACTION ];
			if( !parse[ LMSG_WORLD_ACTION ] )
                break;

            extractStringValue( 16, string1 ); //the world action
            notifyListeners( LMSG_WORLD_ACTION, strcpy( params, UTIL_VarArgs("%s", string1 ) ) );
            break;

        case 'T':
            if( !allowTeam )
                break;

            extractStringValue( 5, string1 ); //the invoking team
            cursor = findForward( 6, 1, '\"');

            switch( message[ cursor + 2 ] )
            {
                case 'f':
                    //alliance
					blockMsg = block[ LMSG_TEAM_ALLIANCE ];
					if( !parse[ LMSG_TEAM_ALLIANCE ] )
                        break;

                    extractStringValue( cursor + 28, string2 ); //the second team
                    notifyListeners( LMSG_TEAM_ALLIANCE, strcpy( params, UTIL_VarArgs("%s %s", string1, string2 ) ) );
                    break;

                case 's':
                    //team score report
					blockMsg = block[ LMSG_TEAM_SCORE ];
					if( !parse[ LMSG_TEAM_SCORE ] )
                        break;

                    extractStringValue( cursor + 9, string2 ); // score
                    cursor = findForward( cursor + 10, 1, '\"' );
                    extractStringValue( cursor + 7, string3 ); //number of players
					loadEnd( findForward( cursor + 7, 1, '(' ), string4 );
					notifyListeners( LMSG_TEAM_SCORE, strcpy( params, UTIL_VarArgs("%s %s %s %s", string1, string2, string3, string4 ) ) );
                    break;

                case 't':
                    //team action
					blockMsg = block[ LMSG_TEAM_ACTION ];
					if( !parse[ LMSG_TEAM_ACTION ] )
                        break;

                    extractStringValue( cursor + 12, string2 ); //team action string
					loadEnd( findForward( cursor + 13, 1, '(' ), string3 );
                    notifyListeners( LMSG_TEAM_ACTION, strcpy( params, UTIL_VarArgs("%s %s %s", string1, string2, string3 ) ) );
                    break;
                default:
                {
                    LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'Team' Switch: %c > %s", message[ cursor + 2 ], message ) );
                }
            }   //end of team
            break;

        case 'L':
            switch( message[ 9 ] )
            {
                case 's':
                    //log starting
					blockMsg = block[ LMSG_LOG ];
					if( !parse[ LMSG_LOG ] )
                        break;

                    loadEnd( 17, string1 );
                    notifyListeners( LMSG_LOG, strcpy( params, UTIL_VarArgs("%c %s", 's', string1 ) ) );
                    break;

                case 'c':
                    //log ending
					blockMsg = block[ LMSG_LOG ];
					if( !parse[ LMSG_LOG ] )
                        break;

                    notifyListeners( LMSG_LOG, strcpy( params, UTIL_VarArgs("%c", 'e' ) ) );
                    break;

                case 'a':
                    //map loading
					blockMsg = block[ LMSG_MAP ];
					if( !parse[ LMSG_MAP ] )
                        break;

                    extractStringValue( 12, string1 ); //map name
                    notifyListeners( LMSG_MAP, strcpy( params, UTIL_VarArgs("%c %s", 'l', string1 ) ) );
                    break;

                default:
                {
                    LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'L' Switch: %c > %s", message[ 9 ], message ) );
                }
            }
            break;

         case 'S':
            switch( message[ 7 ] )
            {

                case ' ':
                    //map start
					blockMsg = block[ LMSG_MAP ];
					if( !parse[ LMSG_MAP ] )
                        break;

                    extractStringValue( 12, string1 ); //map name
                    loadEnd( findForward( 22, 1, '(' ), string2 );
                    notifyListeners( LMSG_MAP, strcpy( params, UTIL_VarArgs("%c %s %s", 's', string1, string2 ) ) );
                    break;

                case 'n':
                    //server name change
					blockMsg = block[ LMSG_SERVER_NAME ];
					if( !parse[ LMSG_SERVER_NAME ] )
                        break;

                    extractStringValue( 15, string1 ); //server name
                    notifyListeners( LMSG_SERVER_NAME, strcpy( params, UTIL_VarArgs("%s", string1 ) ) );
                    break;

                case 's':
                    //server talking
					blockMsg = block[ LMSG_SERVER_SAY ];
					if( !parse[ LMSG_SERVER_SAY ] )
                        break;

                    extractStringValue( 11, string1 ); //message
                    notifyListeners( LMSG_SERVER_SAY, strcpy( params, UTIL_VarArgs("%s", string1 ) ) );
                    break;

                case 'c':
                    //cvar
					blockMsg = block[ LMSG_CVAR ];
					if( !parse[ LMSG_CVAR ] )
                        break;

                    if( message[ 11 ] == ' ' )
                    {
                        extractStringValue( 12, string1 ); //cvar name
                        cursor = findForward( 12, 1, '=' );
                        extractStringValue( cursor + 2, string2 ); //cvar value
                        notifyListeners( LMSG_CVAR, strcpy( params, UTIL_VarArgs("%s %s", string1, string2 ) ) );
                    }
                    else if( message[ 13 ] == 's' )
                    {
                        notifyListeners( LMSG_CVAR, strcpy( params, UTIL_VarArgs("%c", 's' ) ) );
                    }
                    else if( message[ 13 ] == 'e' )
                    {
                        notifyListeners( LMSG_CVAR, strcpy( params, UTIL_VarArgs("%c", 'e' ) ) );
                    }
                    else
                    {
                        LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In Cvar Conds: %s", message ) );
                    }
                    break;


                default:
                {
                    LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Match In 'S' Switch: %c > %s", message[ 7 ], message ) );
                }
            }
            break;

        case 'B':
            //bad rcon -just send the whole thing and let plugins deal with it
			blockMsg = block[ LMSG_RCON ];
			if( !parse[ LMSG_RCON ] )
                break;

            notifyListeners( LMSG_RCON, strcpy( params, UTIL_VarArgs("%s", message ) ) );
            break;

        case 'R':
            //good rcon - just send the whole thing and let plugins deal with it
			blockMsg = block[ LMSG_RCON ];
			if( !parse[ LMSG_RCON ] )
                break;

            notifyListeners( LMSG_RCON, strcpy( params, UTIL_VarArgs("%s", message ) ) );
            break;

        default:
        {
            LOG_MESSAGE(PLID, UTIL_VarArgs( "ERROR: No Recognition: %s", message ) );
        }
    }


    if( blockMsg )
        RETURN_META(MRES_SUPERCEDE);
    else
	    RETURN_META(MRES_HANDLED);
}





/********************************************************************************************************
**************************************       LOG FILE     ***********************************************
*********************************************************************************************************/


/**
*
* - 07/24/2001 04:15AM PST current version: 0.01a pre-release Bud-froggy
*	Log file creation.
*
* - 07/24/2001 09:45PM PST current version: 0.01a pre-release Bud-froggy
*	GPL coverage added.
*
* - 07/25/2001 08:53PM PST current version: 0.01a pre-release Bud-froggy
*	Fixed vers_logd.h.
*
* - 07/26/2001 11:23PM PST current version: 0.01a pre-release Kndroc
*	Added GetUserIndex() Function by:
*		- adding amutil.h
*		- adding amutil.cpp
*		- including <extdll.h>
*
* - 07/27/2001 08:57AM PST current version: 0.01a pre-release Bud-froggy
*	Added FireArms support for the kill message through switch on '(' or 'w'.
*
* - 07/27/2001 12:18PM PST current version: 0.01a pre-release Kndroc
* Changed the Following the in the kill message event code:
*    - added a default to the 'kill' switch to print the unrecognized value
*    - added a call to loadEnd on the FA version of the kill event code so
*          that other info after the weapon could still be passed to the plugins
*    - fixed the cursor position for extracting the weapon name for FA, was 8 now 10
*    - added a check for 'w' after '(' for FA support
* Commented out the rcon decoding and 'L' switch of the main switch
*    since the API doesn't notify us of those either.
*
* - 07/27/2001 08:09PM PST current version: 0.01a pre-release Kndroc
* Updated loadEnd to subsitute the characters '#' and '$' at the correct times instead
*   of using spaces.
*
* - 07/27/2001 10:26PM PST current version: 0.01a pre-release Bud-froggy
* Removed FA support due to non Standard Format logging: no WONID, and weapon
* statement is (weapon "weapname") instead of with "weapname".
*
* - 09/19/2001 09:37AM PST current version 1.0 Kndroc
* . Removed Commented section as they server no purpose (the code doesn't get called
*   anyway if we can't hook the event). Having it active means in the field if something
*   isn't dormant but we thought it was, LogD will start catching it.
* . Changed all SERVER_COMMAND macro calls to the notifyListeners call so listeners can
*   be used.
* . After every 'case' statement a quick boolean switch was added.  If a listener doesn't
*   want a message then the parser can turn off parsing for that event by making the bool
*   false.
*
**/



