// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// log_plugin.cpp - logging routines; should be generic for use in any plugin

/**
 *  Copyright (c) 2001, Will Day <willday@hpgx.net>, Tim Thelin <kndroc@hotmail.com>,
 *  Pieter de Zwart <pdezwart@scu.edu>, David Mart 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   In addition, as a special exception, <name of copyright
 *   holder> gives permission to link the code of this program with
 *   the FOO library (or with modified versions of FOO that use the
 *   same license as FOO), and distribute linked combinations including
 *   the two.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than FOO.  If you modify
 *   this file, you may extend this exception to your version of the
 *   file, but you are not obligated to do so.  If you do not wish to
 *   do so, delete this exception statement from your version.
 *
**/

/**
 * This file is part of Log Daemon.
 *
 *   Log Daemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Log Daemon is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Log Daemon; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

#include <stdio.h>		// vsnprintf(), etc
#include <stdarg.h>		// va_start(), etc

#include <extdll.h>
#include <enginecallback.h>		// ALERT()

#include <meta_api.h>
#include <support_meta.h>

//#include "log_logd.h"
#include "vers_logd.h"

cvar_t logd_version = {"logd_version",MOD_VERSION,FCVAR_SERVER}; 
cvar_t init_plugin_debug = {"logd_debug", "0", FCVAR_EXTDLL, 0, NULL};
cvar_t *plugin_debug;


// Setup for logging.  We pass the name of the debug cvar so that this
// code can be used by multiple different plugins.
void log_init(const char *cvarname) {
	CVAR_REGISTER(&init_plugin_debug);
	CVAR_REGISTER(&logd_version);
	// Since metamod replaces our cvar_t structure with a structure local
	// to metamod when we call CVAR_REGISTER, we have to get the address of
	// the new struct if we expect to be able to read/write directly to
	// the cvar struct.
	plugin_debug=CVAR_GET_POINTER("logd_debug");
	// Since we haven't read server.cfg yet, allow debug logging (level 3)
	// to be enabled with the "+developer" cmdline option.
	if((int) CVAR_GET_FLOAT("developer") != 0)
		CVAR_SET_FLOAT(cvarname, 3);
}

