// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// log_plugin.h - functions and macros for logging

/**
 *  Copyright (c) 2001, Will Day <willday@hpgx.net>, Tim Thelin <kndroc@hotmail.com>,
 *  Pieter de Zwart <pdezwart@scu.edu>, David Mart
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   In addition, as a special exception, <name of copyright
 *   holder> gives permission to link the code of this program with
 *   the FOO library (or with modified versions of FOO that use the
 *   same license as FOO), and distribute linked combinations including
 *   the two.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than FOO.  If you modify
 *   this file, you may extend this exception to your version of the
 *   file, but you are not obligated to do so.  If you do not wish to
 *   do so, delete this exception statement from your version.
 *
**/

/**
 * This file is part of Log Daemon.
 *
 *   Log Daemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Log Daemon is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Log Daemon; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

#ifndef LOG_PLUGIN_H
#define LOG_PLUGIN_H

#include <enginecallback.h>		// ALERT()
#include <sdk_util.h>			// UTIL_VarArgs()

#include <api_info.h>

extern cvar_t init_plugin_debug;
extern cvar_t *plugin_debug;



// called as:
//    LOG_DEBUG(3, ("return code: %d", ret));
#define LOG_DEBUG(level, args) \
	do { if(level <= plugin_debug->value) ALERT(at_logged, "[%s] DEBUG(%d): %s\n", Plugin_info.logtag, level, UTIL_VarArgs args ); } while(0)

// max buffer size for printed messages
#define MAX_LOGMSG_LEN  1024




void log_init(const char *cvarname);

#endif /* LOG_PLUGIN_H */

